@@ -1,6 +1,6 @@
 // libiio-sys/src/channel.rs
 //
-// Copyright (c) 2018-2019, Frank Pagliughi
+// Copyright (c) 2018-2021, Frank Pagliughi
 //
 // Licensed under the MIT license:
 //   <LICENSE or http://opensource.org/licenses/MIT>
@@ -10,16 +10,18 @@
 //! Industrial I/O Channels
 //!
 
-use std::any::TypeId;
-use std::collections::HashMap;
-use std::ffi::CString;
-use std::os::raw::{c_int, c_longlong, c_uint, c_void};
-use std::{mem, str};
-
 use super::*;
-use crate::ffi;
+use crate::{ffi, ATTR_BUF_SIZE};
+use std::{
+    any::TypeId,
+    collections::HashMap,
+    ffi::CString,
+    mem,
+    os::raw::{c_char, c_int, c_longlong, c_uint, c_void},
+};
 
 /// The type of data associated with a channel.
+#[allow(missing_docs)]
 #[repr(u32)]
 #[derive(Debug, Clone, Copy, PartialEq)]
 pub enum ChannelType {
@@ -66,6 +68,7 @@ pub struct DataFormat {
 }
 
 impl DataFormat {
+    /// Creates a new data format from the underlyinh library type.
     fn new(data_fmt: ffi::iio_data_format) -> Self {
         DataFormat { data_fmt }
     }
@@ -150,10 +153,12 @@ impl DataFormat {
 }
 
 /// An Industrial I/O Device Channel
+#[derive(Debug, Clone)]
 pub struct Channel {
+    /// Pointer to the underlying IIO channel object
     pub(crate) chan: *mut ffi::iio_channel,
     #[allow(dead_code)]
-    // looks like it's unused, but really it's holding the Device's lifetime for libiio safety.
+    /// Holder for the Device's lifetime for libiio safety.
     pub(crate) ctx: Context,
 }
 
@@ -184,12 +189,18 @@ impl Channel {
         unsafe { ffi::iio_channel_is_scan_element(self.chan) }
     }
 
+    /// Gets the index of the channel in the device
     pub fn index(&self) -> Result<usize> {
         let ret = unsafe { ffi::iio_channel_get_index(self.chan) };
         sys_result(ret as i32, ret as usize)
     }
 
-    /// Gets the number of context-specific attributes
+    /// Determines if the channel has any attributes
+    pub fn has_attrs(&self) -> bool {
+        unsafe { ffi::iio_channel_get_attrs_count(self.chan) > 0 }
+    }
+
+    /// Gets the number of attributes for the channel
     pub fn num_attrs(&self) -> usize {
         let n = unsafe { ffi::iio_channel_get_attrs_count(self.chan) };
         n as usize
@@ -204,7 +215,7 @@ impl Channel {
     /// Gets the channel-specific attribute at the index
     pub fn get_attr(&self, idx: usize) -> Result<String> {
         let pstr = unsafe { ffi::iio_channel_get_attr(self.chan, idx as c_uint) };
-        cstring_opt(pstr).ok_or_else(|| Error::InvalidIndex)
+        cstring_opt(pstr).ok_or(Error::InvalidIndex)
     }
 
     /// Try to find the channel-specific attribute by name.
@@ -214,6 +225,32 @@ impl Channel {
         cstring_opt(pstr)
     }
 
+    /// Reads a channel-specific attribute
+    ///
+    /// `attr` The name of the attribute
+    pub fn attr_read<T: FromAttribute>(&self, attr: &str) -> Result<T> {
+        let sval = self.attr_read_str(attr)?;
+        T::from_attr(&sval)
+    }
+
+    /// Reads a channel-specific attribute as a string
+    ///
+    /// `attr` The name of the attribute
+    pub fn attr_read_str(&self, attr: &str) -> Result<String> {
+        let mut buf = vec![0 as c_char; ATTR_BUF_SIZE];
+        let attr = CString::new(attr)?;
+        let ret = unsafe {
+            ffi::iio_channel_attr_read(self.chan, attr.as_ptr(), buf.as_mut_ptr(), buf.len())
+        };
+        sys_result(ret as i32, ())?;
+        let s = unsafe {
+            CStr::from_ptr(buf.as_ptr())
+                .to_str()
+                .map_err(|_| Error::StringConversionError)?
+        };
+        Ok(s.into())
+    }
+
     /// Reads a channel-specific attribute as a boolean
     /// `attr` The name of the attribute
     pub fn attr_read_bool(&self, attr: &str) -> Result<bool> {
@@ -277,6 +314,26 @@ impl Channel {
         sys_result(ret, map)
     }
 
+    /// Writes a channel-specific attribute
+    ///
+    /// `attr` The name of the attribute
+    /// `val` The value to write
+    pub fn attr_write<T: ToAttribute>(&self, attr: &str, val: T) -> Result<()> {
+        let sval = T::to_attr(&val)?;
+        self.attr_write_str(attr, &sval)
+    }
+
+    /// Writes a channel-specific attribute as a string
+    ///
+    /// `attr` The name of the attribute
+    /// `val` The value to write
+    pub fn attr_write_str(&self, attr: &str, val: &str) -> Result<()> {
+        let attr = CString::new(attr)?;
+        let sval = CString::new(val)?;
+        let ret = unsafe { ffi::iio_channel_attr_write(self.chan, attr.as_ptr(), sval.as_ptr()) };
+        sys_result(ret as i32, ())
+    }
+
     /// Writes a channel-specific attribute as a boolean
     ///
     /// `attr` The name of the attribute
@@ -316,12 +373,12 @@ impl Channel {
     ///
     /// Before creating a buffer, at least one channel of the device
     /// must be enabled.
-    pub fn enable(&mut self) {
+    pub fn enable(&self) {
         unsafe { ffi::iio_channel_enable(self.chan) };
     }
 
     /// Disable the channel
-    pub fn disable(&mut self) {
+    pub fn disable(&self) {
         unsafe { ffi::iio_channel_disable(self.chan) };
     }
 
@@ -500,14 +557,18 @@ impl Channel {
 }
 
 /// Iterator over the attributes of a Channel
+#[derive(Debug)]
 pub struct AttrIterator<'a> {
+    /// Reference to the Channel that we're scanning for attributes
     chan: &'a Channel,
+    /// Index for the next Channel attribute from the iterator
     idx: usize,
 }
 
 impl<'a> Iterator for AttrIterator<'a> {
     type Item = String;
 
+    /// Gets the next Channel attribute from the iterator
     fn next(&mut self) -> Option<Self::Item> {
         match self.chan.get_attr(self.idx) {
             Ok(name) => {
